#!/usr/bin/python
# -*- mode: python; coding: utf-8 -*-

# Copyright © 2009 by Jeffrey C. Ollie
#
# This is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see
# <http://www.gnu.org/licenses/>.

import sys
import subprocess
import re
import genshi.template

status_map = {None:   3,
              'rc':   2,
              'beta': 1}

class Version:
    def __init__(self, a, b, c):
        self.a = a
        self.cmp = [map(int, a.split('.'))]
        self.b = b
        try:
            self.cmp.append(status_map[b])
        except:
            self.cmp.append(0)
        self.c = c
        if c is None:
            self.cmp.append([])
        else:
            self.cmp.append(map(int, c.split('.')))

    def __cmp__(self, other):
        return cmp(self.cmp, other.cmp)

    def __str__(self):
        if self.b is not None:
            return '%s-%s%s' % (self.a, self.b, self.c)
        return self.a

    def __repr__(self):
        if self.b is not None:
            return '%s-%s%s' % (self.a, self.b, self.c)
        return self.a

baseurl = 'http://svn.digium.com/svn/asterisk'

svn = subprocess.Popen(['/usr/bin/svn', 'ls', '%s/tags' % baseurl],
                       stdin = None, #file('/dev/null', 'r'),
                       stdout = subprocess.PIPE,
                       stderr = subprocess.STDOUT)

data1 = svn.communicate()[0]

svn = subprocess.Popen(['/usr/bin/svn', 'ls', '%s/branches' % baseurl],
                       stdin = None, #file('/dev/null', 'r'),
                       stdout = subprocess.PIPE,
                       stderr = subprocess.STDOUT)

data2 = svn.communicate()[0]

version_re = re.compile('^((\d+(?:\.\d+)+)(?:-(rc|beta)(\d+(?:\.\d+)*))?)/$')

tags = [Version('1.6.0.4', 'rc', '1')]
for line in data1.splitlines():
    match = version_re.match(line)
    if match:
        tag = apply(Version, match.groups()[1:])
        tags.append(tag)

branches = []
for line in data2.splitlines():
    match = version_re.match(line)
    if match:
        branch = apply(Version, match.groups()[1:])
        branches.append(branch)

template = genshi.template.NewTextTemplate("""[core]
\trepositoryformatversion = 0
\tfilemode = true
\tbare = true
\tlogallrefupdates = true
[svn-remote "svn"]
\turl = http://svn.digium.com/svn/asterisk
\tfetch = trunk:refs/remotes/trunk
{% for branch in branches %}\
\tfetch = branches/${branch}:refs/remotes/branches/${branch}
{% end %}\
{% for tag in tags %}\
\tfetch = tags/${tag}:refs/remotes/tags/${tag}
{% end %}\
""")

branches.sort()
tags.sort()

stream = template.generate(tags = tags,
                           branches = branches)
config = stream.render()

sys.stdout.write(config)